<?php

namespace Drupal\commerce_epayco\Plugin\Commerce\PaymentGateway;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\epayco\GatewayHandlerInterface;

/**
 * Provides the One-page payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "epayco_onepage_checkout",
 *   label = "ePayco (One page checkout)",
 *   display_label = "ePayco (One page checkout)",
 *   modes = {
 *     "n/a" = @Translation("N/A"),
 *   },
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_epayco\PluginForm\OffsiteRedirect\OnePageCheckoutForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "amex", "dinersclub", "discover", "jcb", "maestro", "mastercard", "visa",
 *   },
 *   requires_billing_information = FALSE
 * )
 */
class OnePageCheckout extends OffsiteCheckoutBase {

  /**
   * The gateway handler.
   *
   * @var \Drupal\epayco\GatewayHandlerInterface
   */
  protected $gatewayHandler;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, PaymentTypeManager $payment_type_manager, PaymentMethodTypeManager $payment_method_type_manager, TimeInterface $time, LoggerInterface $logger, GatewayHandlerInterface $gateway_handler) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time, $logger);

    $this->gatewayHandler = $gateway_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('datetime.time'),
      $container->get('commerce_epayco.logger'),
      $container->get('epayco.handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['auto_open'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Auto-open payment modal'),
      '#description' => $this->t('Open modal when user arrives to payment process.'),
      '#default_value' => isset($this->configuration['auto_open']) ? $this->configuration['auto_open'] : TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $this->configuration['auto_open'] = $form_state->getValue($form['#parents'])['auto_open'];
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $ref_payco = $request->query->get('ref_payco');
    if ($ref_payco) {
      $info = $this->gatewayHandler->getReferenceRemoteData($ref_payco);
      if (isset($info['data'])) {
        $x_cod_transaction_state = isset($info['data']['x_cod_transaction_state']) ? $info['data']['x_cod_transaction_state'] : '';
        $x_ref_payco = isset($info['data']['x_ref_payco']) ? $info['data']['x_ref_payco'] : '';
        $x_transaction_state = isset($info['data']['x_transaction_state']) ? $info['data']['x_transaction_state'] : '';
        $x_response_reason_text = isset($info['data']['x_response_reason_text']) ? $info['data']['x_response_reason_text'] : '';

        switch ($x_cod_transaction_state) {
          case 1:
            // Aceptada - Accepted.
          case 3:
            // Pendiente - Pending.
            $this->entityTypeManager->getStorage('commerce_payment')->create([
              'state' => $x_cod_transaction_state == 1 ? 'completed' : 'authorization',
              'amount' => $order->getTotalPrice(),
              'payment_gateway' => $this->entityId,
              'order_id' => $order->id(),
              'test' => $this->getMode() == 'test',
              'remote_id' => $x_ref_payco,
              'remote_state' => $x_transaction_state,
              'authorized' => REQUEST_TIME,
            ])->save();
            $this->messenger()->addMessage($this->t('Transaction @status by ePayco.', ['@status' => $x_transaction_state]));
            break;

          case 2:
            // Rechazada - Rejected.
          case 4:
            // Fallida - Failed.
            $this->messenger()->addMessage($this->t('Order was finished, but payment transaction failed. ePayco says: @status', ['@status' => $x_transaction_state]), 'error');
            if (!empty($x_response_reason_text)) {
              $this->logger->log('error', $x_response_reason_text);
            }
            break;
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onCancel(OrderInterface $order, Request $request) {
    $status = $request->get('status');
    $this->messenger()->addMessage($this->t('Payment @status on @gateway but may resume the checkout process here when you are ready.', [
      '@status' => $status,
      '@gateway' => $this->getDisplayLabel(),
    ]), 'error');
  }

}
